part of aargul_crud;

// class InfoLabel extends StatelessWidget {
//   final String? label;
//   final Widget child;
//   const InfoLabel(
//       {super.key,
//       this.label,
//       this.labelWidget,
//       this.labelStyle,
//       required this.child});

//   @override
//   Widget build(BuildContext context) {
//     return Column(
//       // mainAxisAlignment: MainAxisAlignment.start,
//       // crossAxisAlignment: CrossAxisAlignment.start,
//       // mainAxisSize: MainAxisSize.min,
//       children: [
//         if (label != null && label!.isNotEmpty || labelWidget != null)
//           labelWidget ??
//               Text(
//                 label!,
//                 style: labelStyle,
//               ),
//         child
//       ],
//     );
//   }
// }
enum AargulCrudFieldLabelVisibility {
  /// Align the label at the top of field.
  top,

  /// Align the label at the bottom of field.
  bottom,

  /// Hide the label.
  none,
}

extension StringCasingExtension on String {
  String get capitalized =>
      length > 0 ? '${this[0].toUpperCase()}${substring(1).toLowerCase()}' : '';
  String get titleCase => replaceAll(RegExp(' +'), ' ')
      .split(' ')
      .map((str) => str.capitalized)
      .join(' ');
}

abstract class AargulCrudFieldType {
  bool hideLabel = false;
  String get name;
  @override
  String toString() {
    return name;
  }

  Widget? call(String fieldName, AargulCrudField field,
      AargulCrudFieldLabelVisibility labelVisibility) {
    return builder(fieldName, field, labelVisibility);
  }

  Widget? builder(String fieldName, AargulCrudField field,
      AargulCrudFieldLabelVisibility labelVisibility);

  Widget? label(String fieldName, AargulCrudField field) {
    return field.labelWidget ??
        Text(
          field.label ?? fieldName.capitalized,
          style: field.labelStyle,
          textAlign: field.labelAlign,
        );
  }

  Widget? cellBuilder(
      String fieldName, AargulCrudField field, Map<String, dynamic> data);

  AargulCrudFieldType();
  static AargulCrudFieldType get submit => AargulCrudFieldType._submit();
  factory AargulCrudFieldType._submit() {
    return __AargulCrudFieldType("submit", (fieldName, field, labelVisibility) {
      field.hasInput = false;
      ValueNotifier<bool> enabled = ValueNotifier(!field.disabled);
      field.on(_fieldUpdatedEvent, (data) {
        enabled.value = !field.disabled;
        field._value = enabled.value;
      });
      return ValueListenableBuilder(
        valueListenable: enabled,
        builder: (context, value, child) => FilledButton(
          onPressed: value
              ? () {
                  field.disabled = true;
                  field.emit("submit");
                  field.on("submited", (v) {
                    field.disabled = false;
                  });
                }
              : null,
          // focusable: value,
          autofocus: value,
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: !value && true
                ? const SizedBox(
                    height: 24,
                    child: CircularProgressIndicator(),
                  )
                : Text(
                    field.label ?? (fieldName.toUpperCase()),
                    // style: const TextStyle(fontSize: 20),
                  ),
          ),
        ),
      );
    }, cellBuilder: (fieldName, field, values) => null, hideLabel: true);
  }
  static AargulCrudFieldType get custom => AargulCrudFieldType._custom();

  factory AargulCrudFieldType._custom() {
    return __AargulCrudFieldType("custom", (fieldName, field, labelVisibility) {
      field.hasInput = false;
      return null;
    }, cellBuilder: (fieldName, field, values) => null, hideLabel: true);
  }
  static AargulCrudFieldType get input => AargulCrudFieldType._input();
  factory AargulCrudFieldType._input() {
    return __AargulCrudFieldType(
      "input",
      (fieldName, field, labelVisibility) {
        var controller = TextEditingController(text: field.value);
        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) {
            return material.TextFormField(
              controller: controller,
              onChanged: (value) => field.value = value,
              // decoration: fieldDecoration,
              enabled: !field.disabled,
              // placeholder: field.placeholder,
              maxLines: 1,
              minLines: 1,
              maxLength: 250,
            );
          },
        );
      },
    );
  }
  static AargulCrudFieldType get password => AargulCrudFieldType._password();
  factory AargulCrudFieldType._password() {
    return __AargulCrudFieldType(
      "password",
      (fieldName, field, labelVisibility) {
        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) => TextField(
            onChanged: (value) => field.value = value,
            enabled: !field.disabled,
            decoration: InputDecoration(hintText: field.placeholder),
            obscuringCharacter: "*",
          ),
        );
      },
    );
  }
  static AargulCrudFieldType get text => AargulCrudFieldType._text();
  factory AargulCrudFieldType._text() {
    return __AargulCrudFieldType(
      "text",
      (fieldName, field, labelVisibility) {
        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) {
            var controller = TextEditingController(text: field.value);
            return material.TextFormField(
              // decoration: fieldDecoration,
              controller: controller,
              onChanged: (event) {
                field.value = controller.text;
              },

              // onChanged: (value) => field.value = value,
              enabled: !field.disabled,
              // placeholder: field.placeholder,
              onEditingComplete: () {
                field.value = controller.text;
              },
              maxLines: null,
              minLines: 5,
            );
          },
        );
      },
    );
  }
  static AargulCrudFieldType get onOff => AargulCrudFieldType._boolean("onOff");
  static AargulCrudFieldType get boolean => AargulCrudFieldType._boolean();
  factory AargulCrudFieldType._boolean([String name = "boolean"]) {
    return __AargulCrudFieldType(
      name,
      (fieldName, field, labelVisibility) {
        return Center(
          child: Switch(
            //initial value
            value: field.value is bool ? field.value : false,
            textOn: (field.options is Map ? field.options[true] : null) ?? 'On',
            textOff:
                (field.options is Map ? field.options[false] : null) ?? 'Off',
            colorOn:
                ("${field.options is Map ? field.options["onColor"] : null}")
                        .fromCssColor() ??
                    Colors.greenAccent,
            colorOff:
                ("${field.options is Map ? field.options["offColor"] : null}")
                        .fromCssColor() ??
                    Colors.redAccent,
            iconOn: (field.options is Map ? field.options["onIcon"] : null) ??
                Icons.done,
            iconOff: (field.options is Map ? field.options["offIcon"] : null) ??
                Icons.remove_circle_outline,
            textSize: double.tryParse(
                    "${field.options is Map ? field.options["textSize"] : ""}") ??
                16,
            onChanged: (bool state) {
              //Use it to manage the different states
              field.value = state;
            },
          ),
        );
      },
      cellBuilder :(name, field, data){
        var v = field.value is bool ? field.value : false;
        if(v){
          return Icon(Icons.circle, color :  v? ("${field.options is Map ? field.options["onColor"] : null}")
                        .fromCssColor() ??
                    Colors.greenAccent: ("${field.options is Map ? field.options["offColor"] : null}")
                        .fromCssColor() ??
                    Colors.redAccent );
        }
        return null;
      },
    );
  }
  static AargulCrudFieldType get phone => AargulCrudFieldType._phone();
  factory AargulCrudFieldType._phone() {
    return __AargulCrudFieldType(
      "phone",
      (fieldName, field, labelVisibility) {
        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) {
            var controller = TextEditingController(text: field.value);
            return material.TextFormField(
              controller: controller,
              // decoration: fieldDecoration,
              // onChanged: (value) => field.value = value,
              enabled: !field.disabled,
              // placeholder: field.placeholder,
              maxLines: 1,
              minLines: 1,
              inputFormatters: [FilteringTextInputFormatter.digitsOnly],
            );
          },
        );
      },
    );
  }
  static AargulCrudFieldType get email => AargulCrudFieldType._email();
  factory AargulCrudFieldType._email() {
    return __AargulCrudFieldType(
      "email",
      (fieldName, field, labelVisibility) {
        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) {
            var controller = TextEditingController(text: field.value);
            return material.TextFormField(
              controller: controller,
              // decoration: fieldDecoration,
              // onChanged: (value) => field.value = value,
              enabled: !field.disabled,
              // placeholder: field.placeholder,
              maxLines: 1,
              minLines: 1,
            );
          },
        );
      },
    );
  }
  static AargulCrudFieldType get number => AargulCrudFieldType._number();
  factory AargulCrudFieldType._number() {
    return __AargulCrudFieldType(
      "number",
      (fieldName, field, labelVisibility) {
        field.value = num.tryParse("${field.value}") ?? 0;
        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) => SpinBox(
              min: (field.options is Map &&
                          (field.options as Map).containsKey("min")
                      ? double.tryParse("${field.options["min"]}")
                      : null) ??
                  0,
              max: (field.options is Map &&
                          (field.options as Map).containsKey("max")
                      ? double.tryParse("${field.options["max"]}")
                      : null) ??
                  1000000,
              onChanged: (value) => field.value = value,
              value: double.tryParse("$value") ?? 0,
              enabled: !field.disabled,
              readOnly: field.disabled),
        );
      },
    );
  }
  static AargulCrudFieldType get select => AargulCrudFieldType._select();
  factory AargulCrudFieldType._select() {
    return __AargulCrudFieldType(
      "select",
      (fieldName, field, labelVisibility) {
        // field.value = field.value != null && field.value is String ? field.value : null;
        return ValueListenableBuilder(
            valueListenable: field,
            builder: (context, value, child) =>
                DropdownButtonFormField<dynamic>(
                  value: value,
                  items: (field.options as Map).entries.map((e) {
                    return DropdownMenuItem<dynamic>(
                      value: e.key,
                      child: Text(e.value.toString()),
                    );
                  }).toList(),
                  onChanged: field.disabled ? null : (s) => field.value = s,
                ));
      },
    );
  }
  static AargulCrudFieldType get time => AargulCrudFieldType._time();
  factory AargulCrudFieldType._time() {
    return __AargulCrudFieldType(
      "time",
      (fieldName, field, labelVisibility) {
        field.value =
            field.value != null && field.value is String ? field.value : "";
        var controller = TextEditingController(text: field.value);

        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) => TextField(
            controller: controller,
            onTap: () {
              var t =
                  (value ?? "${DateTime.now().hour}:${DateTime.now().minute}")
                      .toString()
                      .split(":")
                      .map(double.tryParse);
              Navigator.of(context).push(
                showPicker(
                  context: context,
                  value: Time(
                      hour:
                          (t.elementAtOrNull(0) ?? DateTime.now().hour).toInt(),
                      minute: (t.elementAtOrNull(1) ?? DateTime.now().hour)
                          .toInt()),
                  sunrise: const TimeOfDay(hour: 6, minute: 0), // optional
                  sunset: const TimeOfDay(hour: 18, minute: 0), // optional
                  duskSpanInMinutes: 120, // optional
                  onChange: field.disabled
                      ? (s) {}
                      : (s) => field.value = "${s.hour}:${s.minute}",
                ),
              );
            },
            readOnly: true,
          ),
        );
      },
    );
  }
  static AargulCrudFieldType get date => AargulCrudFieldType._date();
  factory AargulCrudFieldType._date() {
    return __AargulCrudFieldType(
      "date",
      (fieldName, field, labelVisibility) {
        field.value = field.value != null && field.value is DateTime
            ? field.value
            : null; // initialize data
        var controller = TextEditingController(text: field.value.toString());

        return ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) => TextField(
            controller: controller,
            onTap: () async {
              var date = await showDatePicker(
                context: context,
                initialDate: DateTime.tryParse(value) ?? DateTime.now(),
                firstDate: DateTime.now(),
                lastDate: DateTime.now(),
              );
              field.value = date ?? field.value;
            },
            readOnly: true,
          ),
        );
      },
    );
  }
}

void _setDefaultAargulCrudFields() {}

class __AargulCrudFieldType extends AargulCrudFieldType {
  final String _name;
  final bool _hideLabel;
  @override
  bool get hideLabel => _hideLabel;

  final Widget? Function(String name, AargulCrudField field,
      AargulCrudFieldLabelVisibility labelVisibility) _builder;
  final Widget? Function(
          String name, AargulCrudField field, Map<String, dynamic> data)?
      _cellBuilder;
  __AargulCrudFieldType(this._name, this._builder,
      {final Widget? Function(
              String name, AargulCrudField field, Map<String, dynamic> data)?
          cellBuilder,
      bool hideLabel = false})
      : _cellBuilder = cellBuilder,
        _hideLabel = hideLabel;

  @override
  Widget? builder(String name, AargulCrudField field,
      AargulCrudFieldLabelVisibility labelVisibility) {
    return _builder.call(name, field, labelVisibility);
  }

  @override
  Widget? cellBuilder(
      String fieldName, AargulCrudField field, Map<String, dynamic> data) {
    return _cellBuilder?.call(name, field, data);
  }

  @override
  // TODO: implement name
  String get name => _name;
}
