part of aargul_crud;

class AargulCrudField<T> extends EventEmitter implements ValueNotifier {
  final AargulCrudFieldType type;
  final bool editable;
  final bool listable;
  final T? initialValue;
  final Widget? labelWidget;
  final TextStyle? labelStyle;
  final TextAlign labelAlign;
  final Widget Function(Widget? child, AargulCrudField<T> field,
      AargulCrudFieldLabelVisibility labelVisibility)? customBuilder;
  final void Function(AargulCrudField<T> field)? onChanged;
  final void Function(String fieldName, AargulCrudField<T> field)? onCreate;
  final void Function(String fieldName, AargulCrudField<T> field)? onUpdate;
  final void Function(String fieldName, AargulCrudField<T> field)? onDelete;
  final Widget Function(
          AargulCrudField<T> field, dynamic value, Map<String, dynamic> values)?
      renderCell;
  final Widget Function(AargulCrudField<T> field)? renderColunm;
  final AargulCrudTableColumnSize? columnSize;
  final AargulCrudColumnSize? xs;
  final AargulCrudColumnSize? sm;
  final AargulCrudColumnSize? md;
  final AargulCrudColumnSize? lg;
  final AargulCrudColumnSize? xl;

  final Map<String, AargulCrudField> _fields = {};
  // ignore: prefer_final_fields
  MapNotifier<String, dynamic> _state = MapNotifier(data: <String, dynamic>{});
  MapNotifier<String, dynamic> get state => _state;
  AargulCrudField? _parent;
  ValueNotifier? get _parentNotifier => _parent?.notifier;
  final ValueNotifier<void> notifier = ValueNotifier(null);
  // ignore: prefer_final_fields
  String _fieldName = "";
  String get fieldName => _fieldName;

  dynamic _options = {};
  dynamic get options => _options;
  set options(dynamic options) {
    _options = options;
    notifyListeners();
  }

  bool _hasInput = true;
  bool get hasInput => _hasInput;

  set hasInput(bool hasInput) {
    _hasInput = hasInput;
    notifyListeners();
  }

  bool _disabled = false;
  bool get disabled => _disabled;
  set disabled(bool disabled) {
    _disabled = disabled;
    notifyListeners();
  }

  String? _label;
  String? get label => _label;
  set label(String? label) {
    _label = label;
    notifyListeners();
  }

  dynamic Function(T? value)? _formatter;
  dynamic Function(T? value)? get formatter => _formatter;
  set formatter(dynamic Function(T? value)? formatter) {
    _formatter ??= formatter;
  }

  String? _placeholder;
  String? get placeholder => _placeholder;
  set placeholder(String? placeholder) {
    _placeholder = placeholder;
    notifyListeners();
  }

  T? _value;
  @override
  T? get value => formatter?.call(_value) ?? _value;

  T? get raw => _value;

  @override
  set value(newValue) {
    if (_value == newValue) {
      return;
    }
    _value = newValue;
    notifyListeners();
  }

  AargulCrudField? formField(String fieldName) {
    return _fields[fieldName];
  }

  AargulCrudField({
    String? placeholder,
    String? label,
    AargulCrudFieldType? type,
    bool disabled = false,
    bool hasInput = true,
    dynamic options = const {},
    this.editable = true,
    this.listable = true,
    this.labelAlign = TextAlign.left,
    this.labelWidget,
    this.customBuilder,
    this.labelStyle,
    this.renderCell,
    this.renderColunm,
    this.columnSize,
    this.onChanged,
    this.onCreate,
    this.onUpdate,
    this.onDelete,
    this.xs,
    this.sm,
    this.md,
    this.lg,
    this.xl,
    this.initialValue,
  })  : type = type ?? AargulCrudFieldType.input,
        _label = label,
        _placeholder = placeholder,
        _options = options,
        _hasInput = hasInput,
        _disabled = disabled,
        _value = initialValue;

  AargulCrudField copyWith({
    String? placeholder,
    String? label,
    TextAlign? labelAlign,
    TextStyle? labelStyle,
    Widget? labelWidget,
    bool? disabled,
    bool? hasInput,
    AargulCrudFieldType? type,
    dynamic options,
    bool? editable,
    bool? listable,
    T? initialValue,
    Widget Function(Widget? child, AargulCrudField<T> field,
            AargulCrudFieldLabelVisibility labelVisibility)?
        customBuilder,
    void Function(AargulCrudField<T> field)? onChanged,
    void Function(String fieldName, AargulCrudField<T> field)? onCreate,
    void Function(String fieldName, AargulCrudField<T> field)? onUpdate,
    void Function(String fieldName, AargulCrudField<T> field)? onDelete,
    Widget Function(AargulCrudField<T> field, dynamic value,
            Map<String, dynamic> values)?
        renderCell,
    Widget Function(AargulCrudField<T> field)? renderColunm,
    AargulCrudTableColumnSize? columnSize,
    AargulCrudColumnSize? xs,
    AargulCrudColumnSize? sm,
    AargulCrudColumnSize? md,
    AargulCrudColumnSize? lg,
    AargulCrudColumnSize? xl,
  }) {
    var field = AargulCrudField<T>(
      placeholder: placeholder ?? this.placeholder,
      label: label ?? this.label,
      labelStyle: labelStyle ?? this.labelStyle,
      labelAlign: labelAlign ?? this.labelAlign,
      labelWidget: labelWidget ?? this.labelWidget,
      disabled: disabled ?? this.disabled,
      hasInput: hasInput ?? this.hasInput,
      type: type ?? this.type,
      options: options ?? this.options,
      editable: editable ?? this.editable,
      listable: listable ?? this.listable,
      customBuilder: customBuilder ?? this.customBuilder,
      renderCell: renderCell ?? this.renderCell,
      renderColunm: renderColunm ?? this.renderColunm,
      columnSize: columnSize ?? this.columnSize,
      onChanged: onChanged ?? this.onChanged,
      onCreate: onCreate ?? this.onCreate,
      onUpdate: onUpdate ?? this.onUpdate,
      onDelete: onDelete ?? this.onDelete,
      xs: xs ?? this.xs,
      sm: sm ?? this.sm,
      md: md ?? this.md,
      lg: lg ?? this.lg,
      xl: xl ?? this.xl,
      initialValue: initialValue ?? this.initialValue,
    );
    field._value = _value;
    return field;
  }

  /// IMPLEMENT VALUE LISTANABLE
  final Map<VoidCallback, EventListener> _listeners = {};
  Timer _notifyListeners = Timer(Duration.zero, () {});
  @override
  void addListener(VoidCallback listener) {
    _listeners[listener]?.cancel();
    _listeners[listener] = on(_fieldUpdatedEvent, (s) => listener());
  }

  @override
  void dispose() {
    for (var value in listeners) {
      value.cancel();
    }
    _listeners.clear();
    listeners.clear();
  }

  @override
  bool get hasListeners => _listeners.isNotEmpty;

  @override
  void notifyListeners() {
    _notifyListeners.cancel();
    _notifyListeners = Timer(const Duration(milliseconds: 15), () {
      if (_fieldName.isNotEmpty) {
        if (hasInput) {
          _state[_fieldName] = value;
        } else {
          _state.remove(_fieldName);
        }
      }
      // external notifier
      thread(() {
        _parentNotifier?.notifyListeners(); // notify the field parent
      })
          .then((value) => thread(() {
                _parent?._parentNotifier
                    ?.notifyListeners(); // notify the field parent
              }))
          .then((v) => thread(() {
                notifier.notifyListeners(); // notify for external handler
              }))
          .then((v) => thread(() {
                onChanged?.call(this); // notify the changement
              }))
          .then((v) => thread(() {
                emit(_fieldUpdatedEvent);
              }));
    });
  }

  @override
  void removeListener(VoidCallback listener) {
    _listeners[listener]?.cancel();
    _listeners.remove(listener);
  }
}
