part of aargul_crud;

// ignore: must_be_immutable
class AargulCrudFieldsWidget extends StatelessWidget {
  Widget _fields = const SizedBox.shrink();
  Widget get fields => _fields;
  final bool hideLabels;
  Widget? _header;
  Widget? get headerWidget => _header;
  Widget? _footer;
  Widget? get footerWidget => _footer;
  final Widget? Function(Map<String, AargulCrudField<dynamic>> fields,
      AargulCrudFieldsWidget? group)? header;
  final Widget? Function(Map<String, AargulCrudField<dynamic>> fields,
      AargulCrudFieldsWidget? group)? footer;
  final AargulCrudColumnSize? xs;
  final AargulCrudColumnSize? sm;
  final AargulCrudColumnSize? md;
  final AargulCrudColumnSize? lg;
  final AargulCrudColumnSize? xl;
  final List<String> fieldsNames;
  AargulCrudFieldsWidget({
    required this.fieldsNames,
    super.key,
    this.hideLabels = false,
    this.header,
    this.footer,
    this.xs,
    this.sm,
    this.md,
    this.lg,
    this.xl,
  });

  @override
  Widget build(BuildContext context) {
    return ListView(
      shrinkWrap: true,
      children: [
        if (_header != null) _header!,
        _fields,
        if (_footer != null) _footer!,
      ],
    );
  }
}

// ignore: must_be_immutable
class AargulCrudFieldsContainer extends AargulCrudFieldsWidget {
  final Decoration? decoration;
  final Color? color;
  final Clip clip;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final Decoration? foregroundDecoration;

  AargulCrudFieldsContainer({
    super.key,
    required super.fieldsNames,
    this.decoration,
    this.color,
    this.margin = const EdgeInsets.all(8),
    this.padding = EdgeInsets.zero,
    this.clip = Clip.none,
    this.foregroundDecoration,
    super.hideLabels,
    super.header,
    super.footer,
    super.xs,
    super.sm,
    super.md,
    super.lg,
    super.xl,
  });

  @override
  Widget build(BuildContext context) {
    return Container(
      clipBehavior: clip,
      margin: margin,
      padding: padding,
      decoration: decoration ?? BoxDecoration(color: color),
      foregroundDecoration: foregroundDecoration,
      child: ListView(
        shrinkWrap: true,
        children: [
          if (_header != null) _header!,
          _fields,
          if (_footer != null) _footer!,
        ],
      ),
    );
  }
}

// ignore: must_be_immutable
class AargulCrudFieldsCard extends AargulCrudFieldsWidget {
  final Color? color;
  final Clip clip;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  AargulCrudFieldsCard({
    super.key,
    required super.fieldsNames,
    this.color,
    this.margin = const EdgeInsets.all(8),
    this.padding = const EdgeInsets.all(8),
    this.clip = Clip.none,
    super.hideLabels,
    super.header,
    super.footer,
    super.xs,
    super.sm,
    super.md,
    super.lg,
    super.xl,
  });

  @override
  Widget build(BuildContext context) {
    return Card(
      color: color,
      clipBehavior: clip,
      margin: margin,
      child: Padding(
        padding: padding,
        child: ListView(
          shrinkWrap: true,
          children: [
            if (_header != null) _header!,
            _fields,
            if (_footer != null) _footer!,
          ],
        ),
      ),
    );
  }
}

// ignore: must_be_immutable
class AargulCrudFieldsExpendable extends AargulCrudFieldsWidget {
  final Color? color;
  final Clip clip;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  ValueNotifier<bool> customTileExpanded = ValueNotifier(false);

  AargulCrudFieldsExpendable({
    super.key,
    required super.fieldsNames,
    this.color,
    this.margin = const EdgeInsets.all(8),
    this.padding = const EdgeInsets.all(8),
    this.clip = Clip.none,
    super.hideLabels,
    super.header,
    super.footer,
    super.xs,
    super.sm,
    super.md,
    super.lg,
    super.xl,
  });

  @override
  Widget build(BuildContext context) {
    return Card(
      color: color,
      clipBehavior: clip,
      margin: margin,
      child: Padding(
        padding: padding,
        child: ValueListenableBuilder(
          valueListenable: customTileExpanded,
          builder: (context, expanded, child) => ExpansionTile(
            title: _header ?? const SizedBox.shrink(),
            maintainState: true,
            trailing: Icon(
              expanded ? Icons.arrow_drop_down_circle : Icons.arrow_drop_down,
            ),
            children: [
              _fields,
              if (_footer != null) _footer!,
            ],
            onExpansionChanged: (bool expanded) {
              customTileExpanded.value = expanded;
            },
          ),
        ),
      ),
    );
  }
}

extension ExtensionAargulCrudManagerForm on AargulCrudManager {
  SpGridItem _formItem(AargulCrudField field, Widget child) {
    return SpGridItem(
      xl: field.xl != null ? (field.xl!.index + 1) : null,
      lg: field.lg != null ? (field.lg!.index + 1) : null,
      md: field.md != null ? (field.md!.index + 1) : null,
      sm: field.sm != null ? (field.sm!.index + 1) : null,
      xs: field.xs != null ? (field.xs!.index + 1) : 12,
      child: child,
    );
  }

  Widget form(
      {Key? key,
      Map<String, dynamic>? values,
      bool? edit,
      AargulCrudField? parent,
      bool shrinkWrap = false,
      List<AargulCrudFieldsWidget> groups = const [],
      BoxDecoration? fieldDecoration,
      AargulCrudFieldLabelVisibility labelVisibility =
          AargulCrudFieldLabelVisibility.top,
      EdgeInsetsGeometry? padding,
      EdgeInsetsGeometry? margin,
      double spacing = 10,
      double runSpacing = 10,
      Map<String, AargulCrudField<dynamic>> footerFields = const {},
      Map<String, AargulCrudField<dynamic>> headerFields = const {},
      Widget? footer,
      Widget? header,
      Function(Object? values)? onSubmit,
      Function(Map<String, dynamic>? values)? onChanged}) {
    bool update = edit ?? values != null;

    MapNotifier<String, dynamic> formValues = MapNotifier(data: values ?? {});
    formValues.addListener(() {
      onChanged?.call(formValues.value);
    });
    // init STATE
    options.fields.forEach((fieldName, field) {
      field._fieldName = fieldName;
      if (formValues.containsKey(fieldName)) {
        field._value = formValues[fieldName];
      }
      field._parent = parent;
      field._state = formValues;
      field._fields.clear();
      field._fields.addAll(options
          .fields); // assign form fields, accessible via field.formFiel(key)
    });

    SpGrid getFields(
        BuildContext context, Map<String, AargulCrudField<dynamic>> fields,
        [bool hasGroup = false]) {
      Timer submit = Timer(Duration.zero, () {});
      List<AargulCrudFieldsWidget> fieldsGroups = const [];
      if (!hasGroup || groups.isEmpty) {
        fieldsGroups = [AargulCrudFieldsWidget(fieldsNames: fields.keys.toList())];
      } else {
        fieldsGroups = groups;
      }
      List<SpGridItem> listFields(Map<String, AargulCrudField<dynamic>> fields,
          [AargulCrudFieldsWidget? group]) {
        return (
            group != null ? group.fieldsNames.map((key) => fields.containsKey(key) && fields[key] != null? MapEntry(key, fields[key]!)  : null).whereType<MapEntry<String, AargulCrudField<dynamic>>>() : fields.entries
          )
            .map((entry) {
              var fieldName = entry.key;
              var field = entry.value;
              if (field.editable == false) {
                return MapEntry<String, dynamic>(fieldName, null);
              }
              var widget = field.type(fieldName, field, labelVisibility);
              widget = field.customBuilder == null
                  ? widget
                  : field.customBuilder?.call(
                      field.type(fieldName, field, labelVisibility),
                      field,
                      labelVisibility);
              if (widget == null) {
                return MapEntry<String, dynamic>(fieldName, null);
              }
              // ignore: no_leading_underscores_for_local_identifiers
              var $l = field.type.label(fieldName, field);
              var label = [
                if ($l != null) $l,
                if ($l != null)
                  SizedBox.square(
                    dimension: spacing,
                  ),
                widget,
              ];
              if (label.length > 1 &&
                  (labelVisibility.index == 1 || labelVisibility.index == 3)) {
                label = label.reversed.toList();
              }
              var item = _formItem(
                field,
                (group?.hideLabels ?? false) ||
                        label.length == 1 ||
                        field.type.hideLabel ||
                        labelVisibility == AargulCrudFieldLabelVisibility.none
                    ? widget
                    : Column(
                        mainAxisSize: MainAxisSize.min,
                        // mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        children: label,
                      ),
              );
              field.on("submit", (v) {
                submit.cancel();
                // DEFER EXECUTION
                submit = Timer(const Duration(milliseconds: 1), () {
                  compute((c) async {
                    await Future.wait(fields.entries.map((field) async {
                      ((update ? field.value.onUpdate : field.value.onCreate) ??
                              ((value, field) async {}))
                          .call(field.key, field.value);
                    }));
                    thread(() async {
                      return ((update ? options.update : options.create) ??
                          ((Map<String, dynamic> value) async {
                            return "";
                          })).call(await compute<Map<String, dynamic>,Map<String, dynamic>>((object)async {
                            return jsonDecode(jsonEncode(object));
                          },formValues.value));
                    }).then((value) async {
                      onSubmit?.call(value);
                    });
                    return null;
                  }, null);
                });
                field.emit("submited", v);
              });
              return item;
            })
            .whereType<SpGridItem>()
            .toList();
      }

      return SpGrid(
        width: MediaQuery.of(context).size.width,
        children: fieldsGroups.map(
          (group) {
            if (group.header != null) {
              group._header = ValueListenableBuilder(
                valueListenable: formValues,
                builder: (context, value, child) =>
                    group.header?.call(fields, group) ??
                    const SizedBox.shrink(),
              );
            }
            if (group.footer != null) {
              group._footer = ValueListenableBuilder(
                valueListenable: formValues,
                builder: (context, value, child) =>
                    group.footer?.call(fields, group) ??
                    const SizedBox.shrink(),
              );
            }

            group._fields = SpGrid(
              width: MediaQuery.of(context).size.width,
              spacing: spacing,
              runSpacing: runSpacing,
              children: listFields(fields, group),
            );
            return SpGridItem(
                sm: (group.sm != null ? (group.sm!.index + 1) : null),
                md: (group.md != null ? (group.md!.index + 1) : null),
                lg: (group.lg != null ? (group.lg!.index + 1) : null),
                xl: (group.xl != null ? (group.xl!.index + 1) : null),
                xs: (group.xs != null ? (group.xs!.index + 1) : null) ?? 12,
                child: group);
          },
        ).toList(),
      );
    }

    return Builder(
      key: key ?? UniqueKey(),
      builder: (context) => Form(
        child: Container(
          margin: margin,
          padding: padding,
          child: shrinkWrap
              ? ListView(
                  shrinkWrap: shrinkWrap,
                  children: [
                    if (header != null) header,
                    if (headerFields.isNotEmpty)
                      getFields(context, headerFields),
                    getFields(context, options.fields, true),
                    if (footerFields.isNotEmpty)
                      getFields(context, footerFields),
                    if (footer != null) footer,
                  ],
                )
              : Column(
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    if (header != null) header,
                    if (headerFields.isNotEmpty)
                      getFields(context, headerFields),
                    Expanded(
                      child: SingleChildScrollView(
                        child: getFields(context, options.fields, true),
                      ),
                    ),
                    if (footerFields.isNotEmpty)
                      getFields(context, footerFields),
                    if (footer != null) footer,
                  ],
                ),
        ),
      ),
    );
  }
}
