part of aargul_crud;

enum AargulCrudTableColumnSize { S, M, L }

extension ExtensionAargulCrudManagerTable on AargulCrudManager {
  Widget table({
    List<AargulCrudManagerAction> actions = const [],
    List<AargulCrudManagerAction> Function(Map<String, dynamic>)? actionsBuilder
  }) {
    return AsyncPaginatedDataTable2(
        source: _DataSourceAsync(options, actions),
        columns: [
          ...options.fields
              .map((fieldName, field) {
                return MapEntry(
                    fieldName,
                    field.listable
                        ? DataColumn2(
                            size: ColumnSize.values[
                                field.columnSize?.index ?? ColumnSize.M.index],
                            label: field.renderColunm?.call(field) ??
                                Text(field.label ?? (fieldName.toUpperCase())),
                          )
                        : null);
              })
              .values
              .whereType<DataColumn2>()
              .toList(),
          if (actions.isNotEmpty)
            const DataColumn2(
              size: ColumnSize.S,
              label: SizedBox.shrink(),
            ),
        ]);
  }
}

class _DataSourceAsync extends AsyncDataTableSource {
  final AargulCrudOptions options;
  final List<AargulCrudManagerAction> actions;
  _DataSourceAsync(this.options, this.actions);

  @override
  Future<AsyncRowsResponse> getRows(int startIndex, int count) async {
    assert(startIndex >= 0);
    AargulCrudDataResponse totalRecords =
        await (options.read?.call(startIndex, count) ??
            Future.value(AargulCrudDataResponse(0, [])));
    var r = AsyncRowsResponse(
        totalRecords.totalRows,
        totalRecords.rows.mapIndexed((rowIndex,row) {
          var key = options.getKey != null
              ? ValueKey<String>(options.getKey!.call(row))
              : null;

          return material.DataRow(
            color : rowIndex%2 != 0 ? MaterialStateProperty.resolveWith<Color?>((Set<MaterialState> states) {
              if (states.contains(MaterialState.selected)) {
                return Colors.black.withOpacity(0.26);
              }
                return Colors.black.withOpacity(0.03);  // Use the default value.
            }) : MaterialStateProperty.resolveWith<Color?>((Set<MaterialState> states) {
              if (states.contains(MaterialState.selected)) {
                return Colors.white.withOpacity(0.40);
              }
                return Colors.white.withOpacity(0.2);  // Use the default value.
            }),
            key: key,
            // selected: key != null
            //     ? selectionRowKeys.where((k) => k == key).isNotEmpty
            //     : false,
            // onSelectChanged: (value) {
            //   if (value != null && key != null) {
            //     setRowSelection(key, value);
            //   }
            // },
            cells: [
              ...options.fields
                  .map((fieldName, field) {
                    return MapEntry(
                      fieldName,
                      !field.listable
                          ? null
                          : material.DataCell(
                              field.renderCell
                                      ?.call(field, row[fieldName], row) ??
                                  field.type.cellBuilder
                                      .call(fieldName, field, row) ??
                                  Text("${row[fieldName] ?? ""}"),
                            ),
                    );
                  })
                  .values
                  .whereType<material.DataCell>()
                  .toList(),
              if (actions.isNotEmpty)
                material.DataCell(Builder(
                  builder: (context) {
                    int index = 0;
                    return PopupMenuButton<int>(
                      itemBuilder: (BuildContext context) {
                        return actions
                            .map(
                              (action) => PopupMenuItem(
                                value: index++,
                                enabled: action.onPressed != null,
                                onTap: () {
                                  action.onPressed?.call(row);
                                },
                                child: DecoratedBox(
                                  decoration: action.decoration,
                                  child : ListTile(
                                    leading: Icon(action.icon),
                                    title: Text(action.label, style: action.labelStyle,),
                                  )
                                ),
                              ),
                            )
                            .toList();
                      },
                    );
                  },
                )),
            ],
          );
        }).toList());

    return r;
  }
}
